local MINOR_VERSION = tonumber(("$Revision: 2 $"):match("%d+"))
local L = Rock("LibRockLocale-1.0"):GetTranslationNamespace("QuestAgent")
QuestAgent = Rock:NewAddon("QuestAgent", "LibFuBarPlugin-3.0", "LibRockConfig-1.0", "AceComm-3.0", "AceSerializer-3.0", "LibRockDB-1.0", "LibRockEvent-1.0", "LibRockTimer-1.0")
QuestAgent.MINOR_VERSION = MINOR_VERSION
local QuestAgent = QuestAgent
QuestAgent.title = "QuestAgent"
QuestAgent.version = MINOR_VERSION
QuestAgent:SetDatabase("QuestAgentDB")
local tablet = AceLibrary("Tablet-2.0")
LibStub("AceSerializer-3.0"):Embed(QuestAgent)
LibStub("AceComm-3.0"):Embed(QuestAgent)
local compress = LibStub:GetLibrary("LibCompress")

QuestAgent:SetDatabaseDefaults('profile', {
})

local WHT = "|cffffffff";
local BLK = "|cff000000";
local YEL = "|cffffff20";
local RED = "|cffff2020";
local GRN = "|cff20ff20";
local BLU = "|cff2020ff";
local HIL = "|cfffbd284";

local log = {}
local exp = {}
local throttleTime = 60 -- Minimum delay between updates, in seconds
local last = 0
local inThrottle = false
local debug = false
local myquests
local lastxp = -1

function QuestAgent:OnInitialize()
 self:SetFuBarOption("tooltipType", "Tablet-2.0")
 self:SetFuBarOption("configType", "Dewdrop-2.0")
 self:SetFuBarOption("clickableTooltip", true)
 self:SetFuBarOption("hasIcon", false)
 
	local optionsTable = {
		name = "QuestAgent",
		desc = self.notes,
		handler = QuestAgent,
		type = 'group',
		args = {
			test = {
				type = 'toggle',
				name = "Test",
				desc = "Test",
				get = function() return true end,
				set = function() end,
			},
			--seconds = {
				--type = 'toggle',
				--name = L["Show seconds"],
				--desc = L["Show seconds"],
				--get = "IsShowingSeconds",
				--set = "ToggleShowingSeconds",
			--},
			--["local"] = {
				--type = 'toggle',
				--name = L["Local time"],
				--desc = L["Toggle between local time and server time"],
				--get = "IsLocalTime",
				--set = "ToggleLocalTime",
			--},
			--both = {
				--type = 'toggle',
				--name = L["Both times"],
				--desc = L["Toggle between showing two times or just one"],
				--get = "IsBothTimes",
				--set = "ToggleBothTimes",
			--},
			--bubble = {
				--type = 'toggle',
				--name = L["Show day/night bubble"],
				--desc = L["Show the day/night bubble on the upper-right corner of the minimap"],
				--get = "IsShowingBubble",
				--set = "ToggleShowingBubble",
			--},
			--color = {
				--type = 'color',
				--name = COLOR or "Color",
				--desc = L["Set the color of the text"],
				--get = "GetColor",
				--set = "SetColor",
				--disabled = function()
					--return not ClockFu:IsFuBarTextColored()
				--end
			--}
		}
	}
	QuestAgent:SetConfigTable(optionsTable)
	QuestAgent.OnMenuRequest = optionsTable
end

function QuestAgent:OnEnable()
 self:AddTimer(5, "DelayedInit")
end

function QuestAgent:DelayedInit()
 self:AddEventListener("PLAYER_XP_UPDATE", "SendExperienceUpdate")
 self:AddEventListener("QUEST_LOG_UPDATE", "UpdateLog")
 self:AddEventListener("PARTY_MEMBERS_CHANGED", "PartyMembersChanged")
 self:AddEventListener("PLAYER_LEVEL_UP", "UpdateFuBarPlugin")
 self:RegisterComm("QuestAgent-1.0-FullData", "OnCommReceived_FullData")
 self:RegisterComm("QuestAgent-1.0-FullUpdateRequest", "OnCommReceived_FullUpdateRequest")
 self:RegisterComm("QuestAgent-1.0-LatencyRequest", "OnCommReceived_LatencyRequest")
 self:RegisterComm("QuestAgent-1.0-LatencyReply", "OnCommReceived_LatencyReply")
 self:RegisterComm("QuestAgent-1.0-ExperienceUpdate", "OnCommReceived_ExperienceUpdate")
 self:UpdateLog()
 self:UpdateFuBarPlugin()
 if GetNumPartyMembers() > 0 then
  self:SendCommMessage("QuestAgent-1.0-FullUpdateRequest", "", "PARTY")
 end
 --if debug then self:AddPlayer(self:GetMyQuests()) end
end

function QuestAgent:Debug(...)
 if debug then self:Print(...) end
end

local encode_translate = {
  [255] = "\255\001",
  [0] = "\255\002"
}

local function encode_helper(char)
  return encode_translate[char:byte()]
end

local decode_translate = {
  ["\001"] = "\255",
  ["\002"] = "\000"
}

local function decode_helper(text)
  return decode_translate[text]
end

function QuestAgent:Encode(data)
 return data:gsub("([\255%z])", encode_helper)
end

function QuestAgent:Decode(data)
 return data:gsub("\255([\001\002])", decode_helper)
end

function QuestAgent:Compress(data)
 --return data
 return self:Encode(compress:Compress(data))
end

function QuestAgent:Decompress(data)
 --return data
 return compress:Decompress(self:Decode(data))
end

function QuestAgent:PartyMembersChanged()
 lastxp = -1 -- Force an experience update
 self:UpdateLog()
end

function QuestAgent:SendExperienceUpdate()
 lastxp = UnitXP("player")
 local o = {
  [1] = UnitName("player"),
  [2] = lastxp,
  [3] = UnitXPMax("player"),
  [4] = UnitLevel("player")
 }
 local s = self:Serialize(o)
 if not s then
  self:print("[SendExperienceUpdate] Error serializing experience information")
  return
 end
 self:SendCommMessage("QuestAgent-1.0-ExperienceUpdate", s, "PARTY")
 self:Debug("Sent experience update. (Size: "..strlen(s)..")")
end

function QuestAgent:SendFullUpdate(type, destination, overrideThrottle)
 if GetTime() - last < throttleTime and not overrideThrottle then
  if not inThrottle then
   inThrottle = true
   self:AddTimer(throttleTime + 0.1, function()
    self:SendFullUpdate(type, destination)
   end)
   self:Debug("Tried to update before throttle expired. Setting up delayed update.")
  else
   self:Debug("Tried to update before throttle expired. Delayed update already set.")
  end
 else
  local o = self:GetMyQuests()
  if not o then
   self:Print("[SendFullUpdate] Error getting quest information")
   return
  end
  local s = self:Serialize(o)
  if not s then
   self:Print("[SendFullUpdate] Error serializing quest information")
   return
  end
  local c = self:Compress(s)
  if not c then
   self:Print("[SendFullUpdate] Error compressing quest information")
   return
  end
  if not type then
   self:Print("[SendFullUpdate] Error: Type not specified")
   return
  end
  if type == "WHISPER" and not destination then
   self:Print("[SendFullUpdate] Error: Destination not specified")
   return
  end
  self:SendCommMessage("QuestAgent-1.0-FullData", c, type, destination)
  self:Debug("Sent message packet. (Normal: "..strlen(s).." / Compressed: "..strlen(c)..")")
  last = GetTime()
  inThrottle = false
 end
end

local lockUpdate = false
function QuestAgent:UpdateLog()
 if lastxp ~= UnitXP("player") then self:SendExperienceUpdate() end
 if not lockUpdate then
  lockUpdate = true
  if GetNumPartyMembers() > 0 then self:SendFullUpdate("PARTY") end
 end
 lockUpdate = false
end

function QuestAgent:OnCommReceived_FullData(prefix, message, distribution, sender)
 local success, o = self:Deserialize(self:Decompress(message))
 if success == false then
  self:Debug("[OnCommReceived_FullData] Failed to deserialize full data update.")
  return
 else
  if o == nil then return end
  if not o.name then return end
  if o.name == (UnitName("player")) then return end
  self:AddPlayer(o)
  if o.timeCreated then
   self:SendCommMessage("QuestAgent-1.0-LatencyRequest", self:Serialize(o.timeCreated), "WHISPER", sender)
   self:Debug("Requesting latency from "..sender)
  end
 end
end

function QuestAgent:OnCommReceived_ExperienceUpdate(prefix, message, distribution, sender)
 local success, o = self:Deserialize(message)
 if success == false then
  self:Debug("[OnCommReceived_ExperienceUpdate] Failed to deserialize experience update.")
  return
 else
  if o == nil then return end
  if not o[1] then return end
  if o[1] == (UnitName("player")) then return end
  exp[o[1]] = o
  self:Debug("Received experience update from "..o[1]..". "..o[4]..": "..o[2].." / "..o[3])
 end
end

function QuestAgent:OnCommReceived_FullUpdateRequest(prefix, message, distribution, sender)
 if sender == (UnitName("player")) then return end
 self:SendExperienceUpdate()
 self:SendFullUpdate("WHISPER", sender, true)
 self:Debug("Received an update request from "..sender)
end

function QuestAgent:OnCommReceived_LatencyRequest(prefix, message, distribution, sender)
 local success, o = self:Deserialize(message)
 if success then
  self:SendCommMessage("QuestAgent-1.0-LatencyReply", self:Serialize(GetTime() - o), "WHISPER", sender)
  self:Debug("Received a latency request from "..sender)
 else
  self:Debug("Received an invalid latency request from "..sender)
 end
end

function QuestAgent:OnCommReceived_LatencyReply(prefix, message, distribution, sender)
 local success, o = self:Deserialize(message)
 if success then
  local k, v
  for k, v in pairs(log) do
   if v.name == sender then
    v.latency = o
   end
  end
  self:UpdateFuBarPlugin()
  self:Debug("Received a latency response from "..sender)
 else
  self:Debug("Received an invalid latency response from "..sender)
 end
end

function QuestAgent:AddPlayer(o)
 if o == nil then return end
 if not o.name then return end
 self:Debug("Updated player "..o.name)
 local isCollapsed = self:RemovePlayer(o.name)
 o.isCollapsed = isCollapsed
 o.lastUpdate = GetTime()
 table.insert(log, o)
 self:UpdateFuBarPlugin()
 qatest = log
end

function QuestAgent:RemovePlayer(name)
 local k, v, isCollapsed
 isCollapsed = false
 for k, v in pairs(log) do
  if v.name == name then
   isCollapsed = v.isCollapsed
   log[k] = nil
  end
 end
 log.deleteTrigger = 0
 log.deleteTrigger = nil
 return isCollapsed
end

function QuestAgent:OnUpdateFuBarText()
 local s = "QuestAgent ("..#log.."/"
 local i = 0
 for k, v in pairs(log) do
  if type(v.name) == "string" and type(v.quests) == "table" then
   i = i + #v.quests
  end
 end
 s = s..i..")"
 self:SetFuBarText(s)
end

function QuestAgent:C(r, g, b)
 return string.format("|cff%02x%02x%02x", (r*255), (g*255), (b*255));
end

function QuestAgent:Print(...) 
	local text = "|cff33ff99QuestAgent|r: "
	for i=1, select("#", ...) do
		text = text..tostring(select(i, ...)).." "
	end
	DEFAULT_CHAT_FRAME:AddMessage(text)
end

function QuestAgent:MakeQuestTag(level, isDaily, questTag, suggestedGroup, colorize)
 if colorize == nil then colorize = true end
 local modifier = ""
 if isDaily then modifier = L["y"]
 elseif questTag == ELITE then modifier = L["+"]
 elseif questTag == LFG_TYPE_DUNGEON then modifier = L["d"]
 elseif questTag == PVP then modifier = L["p"]
 elseif questTag == RAID then modifier = L["r"]
 elseif questTag == GROUP then modifier = L["g"]
 end
 if suggestedGroup and suggestedGroup > 0 then modifier = modifier..suggestedGroup end
 
 local tag = "["
 if colorize then tag = tag..WHT end
 tag = tag..level
 if colorize then tag = tag.."|r"..HIL end
 tag = tag..modifier
 if colorize then tag = tag.."|r" end
 tag = tag.."]"
 return tag
end

function QuestAgent:OthersOnQuest(name, questId)
 local k, v, kk, vv
 local count = 0
 -- Check others in your log
 for k, v in pairs(log) do
  if type(v.name) == "string" and type(v.quests) == "table" then
   if v.name ~= name and v.name ~= UnitName("player") then
    for kk, vv in pairs(v.quests) do
     if vv.questId == questId then
      count = count + 1
     end
    end
   end
  end
 end
 -- Check yourself
 if myquests then
  for k, v in pairs(myquests.quests) do
   if v.questId == questId then
    count = count + 1
   end
  end
 end
 if count == 0 then return "" else return count end
end

function QuestAgent:GetLastUpdate(lastUpdate)
 if GetTime() - lastUpdate >= 1 then
  return "("..SecondsToTime(GetTime() - lastUpdate)..")"
 end
 return L["(Now)"]
end

function QuestAgent:FormatNumber(num)
 return string.format("%d", num)
end

function QuestAgent:FormatSafePercent(a, b)
 local v
 if b == 0 then v = 0 else v = (a / b) * 100 end
 return string.format("%.2f", v).."%"
end

function QuestAgent:OnUpdateFuBarTooltip()
 local cat
 local o = log
 local lastZone = ""
 local added = 0
 
 local k, v
 for k, v in pairs(o) do
  if type(v.name) == "string" and type(v.quests) == "table" then
  
   added = added + 1
 
   local latency = ""
   if v.latency then latency = SecondsToTime(v.latency) end
   
   cat = tablet:AddCategory(
    "columns", 4,
    "hideBlankLine", true,
    "showWithoutChildren", true,
    "checked", true,
    "hasCheck", true,
    "checkIcon", v.isCollapsed and "Interface\\Buttons\\UI-PlusButton-Up" or "Interface\\Buttons\\UI-MinusButton-Up",
    
    "text", v.name.." "..self:GetLastUpdate(v.lastUpdate),
    "text2", "("..#v.quests.." quests)",
    "text3", latency,
    
    "func", "OnQuestClick",
    "arg1", self,
    "arg2", "header",
    "arg3", v
   )
   
   if not v.isCollapsed then
    if exp[v.name] then
     cat:AddLine(
      "text", HIL.."["..WHT..exp[v.name][4].."|r]|r "..exp[v.name][2].." / "..exp[v.name][3],
      "text2", "Remaining: "..self:FormatNumber(exp[v.name][3] - exp[v.name][2]).." ("..self:FormatSafePercent((exp[v.name][3] - exp[v.name][2]), exp[v.name][3])..")",
      "text3", self:FormatSafePercent(exp[v.name][2], exp[v.name][3])
     )
    end
   
    local kk, vv
    for kk, vv in pairs(v.quests) do
    
     local status = ""
     if vv.isComplete == 1 then status = COMPLETE
     elseif vv.isComplete == -1 then status = FAILED
     end
     
     local tag = self:MakeQuestTag(vv.level, vv.isDaily, vv.questTag, vv.suggestedGroup)
          
     local difficultyColor = GetDifficultyColor(vv.level)
     
     local zone = ""
     if lastZone ~= vv.zone then
      zone = vv.zone
      lastZone = zone
     end
     
     cat:AddLine(
      "text", zone, "textR", 0, "textG", 1, "textB", 0, "justify", "LEFT",
      "text2", tag.." "..self:C(difficultyColor.r, difficultyColor.g, difficultyColor.b)..vv.questTitle.."|r", "justify2", "LEFT",
      "text3", status, "justify3", "LEFT",
      "text4", self:OthersOnQuest(v.name, vv.questId), "justify4", "LEFT",
      "func", "OnQuestClick",
      "arg1", self,
      "arg2", "quest",
      "arg3", vv,
      "arg4", k,
      "arg5", kk
     )
    end
   end
  end
 end
 
 if added <= 0 then
  cat = tablet:AddCategory("columns", 1)
  cat:AddLine(
   "text",
   L["Hint: Join a party. If they're using QuestAgent, you'll see their quests after 2-3 minutes."],
   "textR", 0, "textG", 1, "textB", 0,
   "wrap", true
  )
 else
  cat = tablet:AddCategory("columns", 1)
  cat:AddLine(
   "text",
   L["Hint: Click a quest to open the quest log"]..". "..
   L["Shift-Click a quest to link it into chat"]..". "..
   L["Ctrl-Click a player to remove them"]..".",
   "textR", 0, "textG", 1, "textB", 0,
   "wrap", true
  )
 end
end

function QuestAgent:OnFuBarClick(button)
end

function QuestAgent:OnQuestClick(type, data, key1, key2)
 if type == "header" then
  if IsControlKeyDown() then
   self:RemovePlayer(data.name)
   self:UpdateFuBarPlugin()
  else
   data.isCollapsed = not data.isCollapsed
   self:UpdateFuBarTooltip()
  end
 else
  if IsShiftKeyDown() then
   ChatEdit_InsertLink(data.questLink);
  else
   self:ShowQuest(data, log[key1].name)
  end
 end
end

function QuestAgent:GetMyQuests()
 local originalEntry = GetQuestLogSelection()
 local o = { name = UnitName("player"), quests = {}, isCollapsed = false, timeCreated = GetTime() }
 local i = 0
 local zone = ""
 while GetQuestLogTitle(i + 1) ~= nil do
  i = i + 1
  local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily = GetQuestLogTitle(i)
  if not isHeader then

   SelectQuestLogEntry(i)
   local questDescription, questObjectives = GetQuestLogQuestText()
   
   local numObjectives = GetNumQuestLeaderBoards()
   local objectiveList = {}
	  for c = 1, numObjectives, 1 do
		  local text, type, finished = GetQuestLogLeaderBoard(c)
		  table.insert(objectiveList, {
		   text = text,
		   type = type,
		   finished = finished,
		  })
		 end
		 
		 local numChoices = GetNumQuestLogChoices()
		 local choiceList = {}
		 for c = 1, numChoices, 1 do
				local name, texture, numItems, quality, isUsable = GetQuestLogChoiceInfo(c)
				local itemLink = GetQuestLogItemLink("choice", c)
				table.insert(choiceList, {
				 name = name,
				 texture = texture,
				 numItems = numItems,
				 quality = quality,
				 isUsable = isUsable,
				 itemLink = itemLink,
				})
		 end
		 
		 local spellTexture, spellName, spellIsTradeskillSpell, spellIsSpellLearned = GetQuestLogRewardSpell()
		 
		 local numRewards = GetNumQuestLogRewards()
		 local rewardList = {}
		 for c = 1, numRewards, 1 do
			 local name, texture, numItems, quality, isUsable = GetQuestLogRewardInfo(c)
			 local itemLink = GetQuestLogItemLink("reward", c)
			 table.insert(rewardList, {
			  name = name,
			  texture = texture,
			  numItems = numItems,
			  quality = quality,
			  isUsable = isUsable,
				 itemLink = itemLink,
			 })
		 end

   table.insert(o.quests, {
    questId = tonumber(GetQuestLink(i):match(":(%d+):")),
    zone = zone,
    questTitle = questTitle,
    level = level,
    questTag = questTag,
    suggestedGroup = suggestedGroup,
    isComplete = isComplete,
    isDaily = isDaily,
    logIndex = i,
    questDescription = questDescription,
    questObjectives = questObjectives,
    numObjectives = numObjectives,
    objectiveList = objectiveList,
    money = GetQuestLogRequiredMoney(),
    numRewards = numRewards,
	   numChoices = numChoices,
	   moneyReward = GetQuestLogRewardMoney(),
	   honor = GetQuestLogRewardHonor(),
	   playerTitle = GetQuestLogRewardTitle(),
	   isFailed = IsCurrentQuestFailed(),
	   myMoney = GetMoney(),
	   material = GetQuestBackgroundMaterial(),
	   choiceList = choiceList,
	   spellTexture = spellTexture,
	   spellName = spellName,
	   spellIsTradeskillSpell = spellIsTradeskillSpell,
	   spellIsSpellLearned = spellIsSpellLearned,
	   rewardList = rewardList,
	   questLink = GetQuestLink(i),
   })
  else
   zone = questTitle or ""
  end
 end
 SelectQuestLogEntry(originalEntry)
 myquests = o
 return o
end
