local L = Rock("LibRockLocale-1.0"):GetTranslationNamespace("QuestAgent")
local old_questlogframe = QuestLogFrame
local old_togglequestlog = ToggleQuestLog
local old_lightheaded_getcurrentqid
if LightHeaded then old_lightheaded_getcurrentqid = LightHeaded.GetCurrentQID end

function QuestAgent:ShowQuest(q, name)
 -- If the normal quest log is open, close it
 if (old_questlogframe:IsShown()) then
  ToggleQuestLog()
 end

 -- Major hacks to get LightHeaded to attach itself to our quest frame
 QuestLogFrame = QuestAgent_QuestLogFrame
 ToggleQuestLog = QuestAgent_ToggleQuestLog
 self:SetLightHeadedPosition()
 
 -- More hacks to get LightHeaded to show our quest
 do
  local id = q.questId
  if LightHeaded then
   LightHeaded.GetCurrentQID = function()
    return id
   end
  end
 end
 
 QuestAgent_QuestLogTitleText:SetText(name..L["'s"].." "..QUEST_LOG)
 ShowUIPanel(QuestAgent_QuestLogFrame)
 
 if LightHeaded then
  LightHeaded:SelectQuestLogEntry()
 end
 
	local questTitle = q.questTitle;
	if (not questTitle) then
		questTitle = "";
	end
	if (q.isFailed) then
		questTitle = questTitle.." - ("..FAILED..")";
	end
	QuestAgent_QuestLogQuestTitle:SetText(self:MakeQuestTag(q.level, q.isDaily, q.questTag, q.suggestedGroup, false).." "..questTitle);

	local questDescription = q.questDescription;
	local questObjectives = q.questObjectives;
	QuestAgent_QuestLogObjectivesText:SetText(questObjectives);
	
	QuestAgent_QuestLogObjective1:SetPoint("TOPLEFT", "QuestAgent_QuestLogObjectivesText", "BOTTOMLEFT", 0, -10);
	
	local numObjectives = q.numObjectives;
	
	for i=1, numObjectives, 1 do
		local string = getglobal("QuestAgent_QuestLogObjective"..i);
		local text = q.objectiveList[i].text;
		local type = q.objectiveList[i].type;
		local finished = q.objectiveList[i].finished;
		if ( not text or strlen(text) == 0 ) then
			text = type;
		end
		if ( finished ) then
			string:SetTextColor(0.2, 0.2, 0.2);
			text = text.." ("..COMPLETE..")";
		else
			string:SetTextColor(0, 0, 0);
		end
		string:SetText(text);
		string:Show();
		QuestFrame_SetAsLastShown(string, QuestAgent_QuestLogSpacerFrame);
	end

	for i=numObjectives + 1, MAX_OBJECTIVES, 1 do
		getglobal("QuestAgent_QuestLogObjective"..i):Hide();
	end
	-- If there's money required then anchor and display it
	if ( q.money > 0 ) then
		if ( numObjectives > 0 ) then
			QuestAgent_QuestLogRequiredMoneyText:SetPoint("TOPLEFT", "QuestAgent_QuestLogObjective"..numObjectives, "BOTTOMLEFT", 0, -4);
		else
			QuestAgent_QuestLogRequiredMoneyText:SetPoint("TOPLEFT", "QuestAgent_QuestLogObjectivesText", "BOTTOMLEFT", 0, -10);
		end
		
		MoneyFrame_Update("QuestAgent_QuestLogRequiredMoneyFrame", q.money);
		
		if ( q.money > q.myMoney ) then
			-- Not enough money
			QuestAgent_QuestLogRequiredMoneyText:SetTextColor(0, 0, 0);
			SetMoneyFrameColor("QuestAgent_QuestLogRequiredMoneyFrame", 1.0, 0.1, 0.1);
		else
			QuestAgent_QuestLogRequiredMoneyText:SetTextColor(0.2, 0.2, 0.2);
			SetMoneyFrameColor("QuestAgent_QuestLogRequiredMoneyFrame", 1.0, 1.0, 1.0);
		end
		QuestAgent_QuestLogRequiredMoneyText:Show();
		QuestAgent_QuestLogRequiredMoneyFrame:Show();
	else
		QuestAgent_QuestLogRequiredMoneyText:Hide();
		QuestAgent_QuestLogRequiredMoneyFrame:Hide();
	end

	if ( q.suggestedGroup > 0 ) then
		local suggestedGroupString = format(QUEST_SUGGESTED_GROUP_NUM, q.suggestedGroup);
		QuestAgent_QuestLogSuggestedGroupNum:SetText(suggestedGroupString);
		QuestAgent_QuestLogSuggestedGroupNum:Show();
		QuestAgent_QuestLogSuggestedGroupNum:ClearAllPoints();
		if ( q.money > 0 ) then
			QuestAgent_QuestLogSuggestedGroupNum:SetPoint("TOPLEFT", "QuestAgent_QuestLogRequiredMoneyText", "BOTTOMLEFT", 0, -4);
		elseif ( numObjectives > 0 ) then
			QuestAgent_QuestLogSuggestedGroupNum:SetPoint("TOPLEFT", "QuestAgent_QuestLogObjective"..numObjectives, "BOTTOMLEFT", 0, -4);
		else
			QuestAgent_QuestLogSuggestedGroupNum:SetPoint("TOPLEFT", "QuestAgent_QuestLogObjectivesText", "BOTTOMLEFT", 0, -10);
		end
	else
		QuestAgent_QuestLogSuggestedGroupNum:Hide();
	end

	if ( q.suggestedGroup > 0 ) then
		QuestAgent_QuestLogDescriptionTitle:SetPoint("TOPLEFT", "QuestAgent_QuestLogSuggestedGroupNum", "BOTTOMLEFT", 0, -10);
	elseif ( q.money > 0 ) then
		QuestAgent_QuestLogDescriptionTitle:SetPoint("TOPLEFT", "QuestAgent_QuestLogRequiredMoneyText", "BOTTOMLEFT", 0, -10);
	elseif ( numObjectives > 0 ) then
		QuestAgent_QuestLogDescriptionTitle:SetPoint("TOPLEFT", "QuestAgent_QuestLogObjective"..numObjectives, "BOTTOMLEFT", 0, -10);
	else
		QuestAgent_QuestLogDescriptionTitle:SetPoint("TOPLEFT", "QuestAgent_QuestLogObjectivesText", "BOTTOMLEFT", 0, -10);
	end
	if ( questDescription ) then
		QuestAgent_QuestLogQuestDescription:SetText(questDescription);
		QuestFrame_SetAsLastShown(QuestAgent_QuestLogQuestDescription, QuestAgent_QuestLogSpacerFrame);
	end
	local numRewards = q.numRewards;
	local numChoices = q.numChoices;
	local money = q.moneyReward;
	local honor = q.honor;
	local playerTitle = q.playerTitle;

	if ( playerTitle or (numRewards + numChoices + money + honor) > 0 ) then
		QuestAgent_QuestLogRewardTitleText:Show();
		QuestFrame_SetAsLastShown(QuestAgent_QuestLogRewardTitleText, QuestAgent_QuestLogSpacerFrame);
	else
		QuestAgent_QuestLogRewardTitleText:Hide();
	end
	
	local numQuestRewards = numRewards;
	local numQuestChoices = numChoices;
	local numQuestSpellRewards = 0;
	local spacerFrame = QuestAgent_QuestLogSpacerFrame;
	if ( q.spellTexture ) then
		numQuestSpellRewards = 1;
	end

	local totalRewards = numQuestRewards + numQuestChoices + numQuestSpellRewards;
	local questItemName = "QuestAgent_QuestLogItem";
	local material = q.material;
	if ( not material ) then
		material = "Parchment";
	end
	local questItemReceiveText = getglobal("QuestAgent_QuestLogItemReceiveText")
	local honorFrame = getglobal("QuestAgent_QuestLogHonorFrame");
	local moneyFrame = getglobal("QuestAgent_QuestLogMoneyFrame");
	local playerTitleFrame = getglobal("QuestAgent_QuestLogPlayerTitleFrame");
	
	if ( totalRewards == 0 and money == 0 and honor == 0 and not playerTitle ) then
		getglobal("QuestAgent_QuestLogRewardTitleText"):Hide();
	else
		getglobal("QuestAgent_QuestLogRewardTitleText"):Show();
		QuestFrame_SetTitleTextColor(getglobal("QuestAgent_QuestLogRewardTitleText"), material);
		QuestFrame_SetAsLastShown(getglobal("QuestAgent_QuestLogRewardTitleText"), spacerFrame);
	end
	if ( money == 0 ) then
		moneyFrame:Hide();
	else
		moneyFrame:Show();
		QuestFrame_SetAsLastShown(moneyFrame, spacerFrame);
		MoneyFrame_Update("QuestAgent_QuestLogMoneyFrame", money);
	end
	if (honor == 0) then
		honorFrame:Hide();
	else
		honorFrame:Show();
		QuestHonorFrame_Update("QuestAgent_QuestLogHonorFrame", honor);
		QuestFrame_SetAsLastShown(honorFrame, spacerFrame);
	end
	if ( not playerTitle ) then
		playerTitleFrame:Hide();
	else
		local anchorFrame;
		if ( honor ~= 0 ) then
			anchorFrame = honorFrame;
		elseif ( money ~= 0 ) then
			anchorFrame = moneyFrame;
		else
			anchorFrame = getglobal("QuestAgent_QuestLogRewardTitleText");
		end
		playerTitleFrame:SetPoint("TOPLEFT", anchorFrame, "BOTTOMLEFT", 0, -5);
		getglobal("QuestAgent_QuestLogPlayerTitleFrameTitle"):SetText(playerTitle);
		playerTitleFrame:Show();
		QuestFrame_SetAsLastShown(playerTitleFrame, spacerFrame);
	end

	-- Hide unused rewards
	for i=totalRewards + 1, MAX_NUM_ITEMS, 1 do
		getglobal(questItemName..i):Hide();
	end

	local questItem, name, texture, isTradeskillSpell, isSpellLearned, quality, isUsable, numItems = 1;
	local rewardsCount = 0;
	
	-- Setup choosable rewards
	if ( numQuestChoices > 0 ) then
		local itemChooseText = getglobal("QuestAgent_QuestLogItemChooseText");
		itemChooseText:Show();
		QuestFrame_SetTextColor(itemChooseText, material);
		QuestFrame_SetAsLastShown(itemChooseText, spacerFrame);
		
		local index;
		local baseIndex = rewardsCount;
		for i=1, numQuestChoices, 1 do	
			index = i + baseIndex;
			questItem = getglobal(questItemName..index);
			questItem.type = "choice";
			numItems = 1;
			name = q.choiceList[i].name;
			texture = q.choiceList[i].texture;
			numItems = q.choiceList[i].numItems;
			quality = q.choiceList[i].quality;
			isUsable = q.choiceList[i].isUsable;
			questItem:SetID(i);
			questItem.itemLink = q.choiceList[i].itemLink;
			questItem:Show();
			-- For the tooltip
			questItem.rewardType = "item"
			QuestFrame_SetAsLastShown(questItem, spacerFrame);
			getglobal(questItemName..index.."Name"):SetText(name);
			SetItemButtonCount(questItem, numItems);
			SetItemButtonTexture(questItem, texture);
			if ( isUsable ) then
				SetItemButtonTextureVertexColor(questItem, 1.0, 1.0, 1.0);
				SetItemButtonNameFrameVertexColor(questItem, 1.0, 1.0, 1.0);
			else
				SetItemButtonTextureVertexColor(questItem, 0.9, 0, 0);
				SetItemButtonNameFrameVertexColor(questItem, 0.9, 0, 0);
			end
			if ( i > 1 ) then
				if ( mod(i,2) == 1 ) then
					questItem:SetPoint("TOPLEFT", questItemName..(index - 2), "BOTTOMLEFT", 0, -2);
				else
					questItem:SetPoint("TOPLEFT", questItemName..(index - 1), "TOPRIGHT", 1, 0);
				end
			else
				questItem:SetPoint("TOPLEFT", itemChooseText, "BOTTOMLEFT", -3, -5);
			end
			rewardsCount = rewardsCount + 1;
		end
	else
		getglobal("QuestAgent_QuestLogItemChooseText"):Hide();
	end
	
	-- Setup spell rewards
	if ( numQuestSpellRewards > 0 ) then
		local learnSpellText = getglobal("QuestAgent_QuestLogSpellLearnText");
		learnSpellText:Show();
		QuestFrame_SetTextColor(learnSpellText, material);
		QuestFrame_SetAsLastShown(learnSpellText, spacerFrame);

		--Anchor learnSpellText if there were choosable rewards
		if ( rewardsCount > 0 ) then
			learnSpellText:SetPoint("TOPLEFT", questItemName..rewardsCount, "BOTTOMLEFT", 3, -5);
		else
			learnSpellText:SetPoint("TOPLEFT", "QuestAgent_QuestLogRewardTitleText", "BOTTOMLEFT", 0, -5);
		end

  texture = q.spellTexture;
  name = q.spellName;
  isTradeskillSpell = q.spellIsTradeskillSpell;
  isSpellLearned = q.spellIsSpellLearned;
		
		if ( isTradeskillSpell ) then
			learnSpellText:SetText(REWARD_TRADESKILL_SPELL);
		elseif ( not isSpellLearned ) then
			learnSpellText:SetText(REWARD_AURA);
		else
			learnSpellText:SetText(REWARD_SPELL);
		end
		
		rewardsCount = rewardsCount + 1;
		questItem = getglobal(questItemName..rewardsCount);
		questItem:Show();
		-- For the tooltip
		questItem.rewardType = "spell";
		questItem.bookType = BOOKTYPE_SPELL;
		--questItem.spellId = 0
		SetItemButtonCount(questItem, 0);
		SetItemButtonTexture(questItem, texture);
		getglobal(questItemName..rewardsCount.."Name"):SetText(name);
		questItem:SetPoint("TOPLEFT", learnSpellText, "BOTTOMLEFT", -3, -5);
	else
		getglobal("QuestAgent_QuestLogSpellLearnText"):Hide();
	end
	
	-- Setup mandatory rewards
	if ( numQuestRewards > 0 or money > 0) then
		QuestFrame_SetTextColor(questItemReceiveText, material);
		-- Anchor the reward text differently if there are choosable rewards
		if ( numQuestSpellRewards > 0  ) then
			questItemReceiveText:SetText(REWARD_ITEMS);
			questItemReceiveText:SetPoint("TOPLEFT", questItemName..rewardsCount, "BOTTOMLEFT", 3, -5);		
		elseif ( numQuestChoices > 0  ) then
			questItemReceiveText:SetText(REWARD_ITEMS);
			local index = numQuestChoices;
			if ( mod(index, 2) == 0 ) then
				index = index - 1;
			end
			questItemReceiveText:SetPoint("TOPLEFT", questItemName..index, "BOTTOMLEFT", 3, -5);
		else 
			questItemReceiveText:SetText(REWARD_ITEMS_ONLY);
			questItemReceiveText:SetPoint("TOPLEFT", "QuestAgent_QuestLogRewardTitleText", "BOTTOMLEFT", 3, -5);
		end
		questItemReceiveText:Show();
		QuestFrame_SetAsLastShown(questItemReceiveText, spacerFrame);
		-- Setup mandatory rewards
		local index;
		local baseIndex = rewardsCount;
		for i=1, numQuestRewards, 1 do
			index = i + baseIndex;
			questItem = getglobal(questItemName..index);
			questItem.type = "reward";
			numItems = 1;
			name = q.rewardList[i].name;
			texture = q.rewardList[i].texture;
			numItems = q.rewardList[i].numItems;
			quality = q.rewardList[i].quality;
			isUsable = q.rewardList[i].isUsable;
			questItem:SetID(i)
			questItem.itemLink = q.rewardList[i].itemLink;
			questItem:Show();
			-- For the tooltip
			questItem.rewardType = "item";
			QuestFrame_SetAsLastShown(questItem, spacerFrame);
			getglobal(questItemName..index.."Name"):SetText(name);
			SetItemButtonCount(questItem, numItems);
			SetItemButtonTexture(questItem, texture);
			if ( isUsable ) then
				SetItemButtonTextureVertexColor(questItem, 1.0, 1.0, 1.0);
				SetItemButtonNameFrameVertexColor(questItem, 1.0, 1.0, 1.0);
			else
				SetItemButtonTextureVertexColor(questItem, 0.5, 0, 0);
				SetItemButtonNameFrameVertexColor(questItem, 1.0, 0, 0);
			end
			
			if ( i > 1 ) then
				if ( mod(i,2) == 1 ) then
					questItem:SetPoint("TOPLEFT", questItemName..(index - 2), "BOTTOMLEFT", 0, -2);
				else
					questItem:SetPoint("TOPLEFT", questItemName..(index - 1), "TOPRIGHT", 1, 0);
				end
			else
				questItem:SetPoint("TOPLEFT", "QuestAgent_QuestLogItemReceiveText", "BOTTOMLEFT", -3, -5);
			end
			rewardsCount = rewardsCount + 1;
		end
	else	
		questItemReceiveText:Hide();
	end
end

function QuestAgent:QuestLog_OnShow()
	PlaySound("igQuestLogOpen");
end

function QuestAgent:QuestLog_OnHide()
 ToggleQuestLog = old_togglequestlog
 QuestLogFrame = old_questlogframe
 self:SetLightHeadedPosition()
 if LightHeaded then
  LightHeaded.GetCurrentQID = old_lightheaded_getcurrentqid
 end
	PlaySound("igQuestLogClose");
end

function QuestAgent_ToggleQuestLog()
 HideUIPanel(QuestAgent_QuestLogFrame)
 old_togglequestlog()
end

function QuestAgent:SetLightHeadedPosition()
 if not LightHeadedFrame then return end
 LightHeadedFrame:SetParent(QuestLogFrame)
 LightHeadedFrame:SetFrameLevel(0)
 LightHeaded:LockUnlockFrame()
end
